// main.h

// This example was made for STM32F407VG on EasyMx PRO v7 for STM32.
//
// It will demonstrate the use of three tasks
// to simultaneously blink LEDs on several different PORTs.

// Includes

#include "main.h"

// Public Function Definitions

// Task 1 - Blinks PORTB LEDs.
void task1(void *pvParameters)
{
    // Set PORTB pins as digital output.
    GPIO_Digital_Output(&GPIOB_BASE, _GPIO_PINMASK_ALL);

    GPIOB_ODR = 0;
    
    // Blink PORTB LEDs with some delay.
    while (1)
    {
        vTaskDelay(pdMS_TO_TICKS(250));
        GPIOB_ODR = ~GPIOB_ODR;
    }
}

// Task 2 - Blinks PORTC LEDs.
void task2(void *pvParameters)
{
    // Set PORTC pins as digital output.
    GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_ALL);

    GPIOC_ODR = 0;

    // Blink PORTC LEDs with some delay.
    while (1)
    {
        vTaskDelay(pdMS_TO_TICKS(500));
        GPIOC_ODR = ~GPIOC_ODR;
    }
}

// Task 3 - Blinks PORTD LEDs.
void task3(void *pvParameters)
{
    // Set PORTD pins as digital output.
    GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_ALL);

    GPIOD_ODR = 0;

    // Blink PORTD LEDs with some delay.
    while (1)
    {
        vTaskDelay(pdMS_TO_TICKS(1000));
        GPIOD_ODR = ~GPIOD_ODR;
    }
}

// Main function, which creates the tasks and starts the scheduler.
void main()
{
    // Create task 1.
    xTaskCreate(
        (TaskFunction_t)task1,
        "Task 1",
        configMINIMAL_STACK_SIZE,
        NULL,
        10,
        NULL
    );
    
    // Create task 2.
    xTaskCreate(
        (TaskFunction_t)task2,
        "Task 2",
        configMINIMAL_STACK_SIZE,
        NULL,
        10,
        NULL
    );
    
    // Create task 3.
    xTaskCreate(
        (TaskFunction_t)task3,
        "Task 3",
        configMINIMAL_STACK_SIZE,
        NULL,
        10,
        NULL
    );

    // Start the RTOS scheduler.
    vTaskStartScheduler();
    
    // Will never reach here.
    while (1);
}